@@ -25,15 +25,15 @@
 #include <libhildondesktop/libhildondesktop.h>
 #include <stdlib.h>
 #include "lib-currency-home-widget.h"
-#include <conic/conic.h>
 #include <string.h>
+#include <pthread.h>
 
 extern int n_currencies;
 extern char *currencies[];
 extern char *countries[];
 
 
-gboolean update_values(CurrencyWidget *desktop_plugin);
+gboolean as_update_values(CurrencyWidget *desktop_plugin);
 
 static GtkWidget *update_ui(CurrencyWidget *plug) {
 
@@ -48,20 +48,31 @@ static GtkWidget *update_ui(CurrencyWidget *plug) {
    
    
     gtk_label_set_markup(GTK_LABEL(plug->info_label), tmp_string);
-    //gtk_label_set_text(plug->info_label, tmp_string);
 
-    gtk_widget_destroy(plug->curr_flag);
+    if ( plug->curr_flag != NULL ) {
+        gtk_widget_destroy(plug->curr_flag);
+    }
     plug->curr_flag = gtk_image_new_from_file (plug->curset->active_currency->icon);
     gtk_container_add(GTK_CONTAINER(plug->curr_box), plug->curr_flag);
     gtk_box_reorder_child (GTK_BOX(plug->curr_box), GTK_WIDGET(plug->curr_flag), 0);
 
     gtk_widget_show_all(GTK_WIDGET(plug->contents));
-
 } 
 
+void set_gui_update_mode(CurrencyWidget *plug) {
+    
+    gtk_label_set_markup(GTK_LABEL(plug->info_label), "updating ....");
+    if ( plug->curr_flag != NULL) {
+      gtk_widget_destroy(plug->curr_flag);
+    }
+
+    gtk_widget_show_all(GTK_WIDGET(plug->contents));
+}
+
 void  build_ui (CurrencyWidget *plug)
 {
 
+    
     GtkWidget *box = GTK_VBOX (gtk_vbox_new (TRUE,5));
     plug->contents =  gtk_vbox_new(TRUE, 5);
 
@@ -102,7 +113,7 @@ callback( GtkWidget *widget,   GdkEventButton* event, CurrencyWidget *desktop_pl
 
     int i,j;
 
-    int  currency_changed = 0;
+    desktop_plugin->curset->currency_changed = 0;
 
     GtkWidget *dialog;
     char tmp[100];
@@ -282,13 +293,13 @@ callback( GtkWidget *widget,   GdkEventButton* event, CurrencyWidget *desktop_pl
                 if ( j == 0 ) {
                     if ( desktop_plugin->curset->type != hildon_picker_button_get_active(HILDON_PICKER_BUTTON(button[j])) + 1 ) {
                         desktop_plugin->curset->type = hildon_picker_button_get_active(HILDON_PICKER_BUTTON(button[j])) + 1;
-                        currency_changed = 1;
+                        desktop_plugin->curset->currency_changed = 1;
                     }
                 }
                 else {
                     if ( desktop_plugin->curset->active_currency->type != hildon_picker_button_get_active(HILDON_PICKER_BUTTON(button[j])) +1) {
                         desktop_plugin->curset->active_currency->type = hildon_picker_button_get_active(HILDON_PICKER_BUTTON(button[j])) + 1;
-                        currency_changed = 1;
+                        desktop_plugin->curset->currency_changed = 1;
                     }
                 }
             }
@@ -298,42 +309,42 @@ callback( GtkWidget *widget,   GdkEventButton* event, CurrencyWidget *desktop_pl
                     if ( desktop_plugin->curset->update_interval != 1 ) {
                         desktop_plugin->curset->update_interval = 1;
                         g_source_remove(desktop_plugin->timeout);
-                        desktop_plugin->timeout = g_timeout_add(TO_HOUR(desktop_plugin->curset->update_interval), update_values,desktop_plugin);
+                        desktop_plugin->timeout = g_timeout_add(TO_HOUR(desktop_plugin->curset->update_interval), as_update_values,desktop_plugin);
                     }
                     break;
                 case 1:
                     if ( desktop_plugin->curset->update_interval != 2 ) {
                         desktop_plugin->curset->update_interval = 2;
                         g_source_remove(desktop_plugin->timeout);
-                        desktop_plugin->timeout = g_timeout_add(TO_HOUR(desktop_plugin->curset->update_interval), update_values, desktop_plugin);
+                        desktop_plugin->timeout = g_timeout_add(TO_HOUR(desktop_plugin->curset->update_interval), as_update_values, desktop_plugin);
                     }
                     break;
                 case 2:
                     if ( desktop_plugin->curset->update_interval != 4 ) {
                         desktop_plugin->curset->update_interval = 4;
                         g_source_remove(desktop_plugin->timeout);
-                        desktop_plugin->timeout = g_timeout_add(TO_HOUR(desktop_plugin->curset->update_interval), update_values, desktop_plugin);
+                        desktop_plugin->timeout = g_timeout_add(TO_HOUR(desktop_plugin->curset->update_interval), as_update_values, desktop_plugin);
                     }
                     break;
                 case 3:
                     if ( desktop_plugin->curset->update_interval != 8 ) {
                         desktop_plugin->curset->update_interval = 8;
                         g_source_remove(desktop_plugin->timeout);
-                        desktop_plugin->timeout = g_timeout_add(TO_HOUR(desktop_plugin->curset->update_interval), update_values, desktop_plugin);
+                        desktop_plugin->timeout = g_timeout_add(TO_HOUR(desktop_plugin->curset->update_interval), as_update_values, desktop_plugin);
                     }
                     break;
                 case 4:
                     if ( desktop_plugin->curset->update_interval != 16 ) {
                         desktop_plugin->curset->update_interval = 16;
                         g_source_remove(desktop_plugin->timeout);
-                        desktop_plugin->timeout = g_timeout_add(TO_HOUR(desktop_plugin->curset->update_interval), update_values, desktop_plugin);
+                        desktop_plugin->timeout = g_timeout_add(TO_HOUR(desktop_plugin->curset->update_interval), as_update_values, desktop_plugin);
                     }
                     break;
                 case 5:
                     if ( desktop_plugin->curset->update_interval != 24 ) {
                         desktop_plugin->curset->update_interval = 24;
                         g_source_remove(desktop_plugin->timeout);
-                        desktop_plugin->timeout = g_timeout_add(TO_HOUR(desktop_plugin->curset->update_interval), update_values, desktop_plugin);
+                        desktop_plugin->timeout = g_timeout_add(TO_HOUR(desktop_plugin->curset->update_interval), as_update_values, desktop_plugin);
                     }
                     break;
                 case 6:
@@ -348,17 +359,7 @@ callback( GtkWidget *widget,   GdkEventButton* event, CurrencyWidget *desktop_pl
                     break;
             }
 
-
-            // Get and update values and gui
-            pthread_mutex_lock(&desktop_plugin->mutex);
-            get_values_json(&desktop_plugin->curset);
-            if ( desktop_plugin->curset->not_updated == 1 && currency_changed == 1) {
-                strcpy(desktop_plugin->curset->active_currency->value, "offline");
-            }
-            save_config(&desktop_plugin);               
-            update_ui(desktop_plugin);
-            pthread_mutex_unlock(&desktop_plugin->mutex);
-
+            as_update_values(desktop_plugin);
             gtk_widget_destroy(dialog);
 
             break;
@@ -374,19 +375,40 @@ callback( GtkWidget *widget,   GdkEventButton* event, CurrencyWidget *desktop_pl
     } } 
 
 
-gboolean update_values(CurrencyWidget *desktop_plugin) {
-   
-    pthread_mutex_lock(&desktop_plugin->mutex);
-    
-    get_values_json(&desktop_plugin->curset);
-    update_ui(desktop_plugin);
 
+void thread_update_values(CurrencyWidget *desktop_plugin) {
+    pthread_mutex_lock(&desktop_plugin->mutex);
+    get_values_json(&desktop_plugin->curset);
+    if ( desktop_plugin->curset->not_updated == 1 && desktop_plugin->curset->currency_changed == 1) {
+        strcpy (desktop_plugin->curset->active_currency->value, "offline");
+    } 
+    
+    update_ui(desktop_plugin);
     pthread_mutex_unlock(&desktop_plugin->mutex);
+    pthread_exit(0);
+}
+   
+gboolean as_update_values(CurrencyWidget *desktop_plugin) {
+  
+    pthread_t tid;
+    int error;
+            
+    pthread_mutex_lock(&desktop_plugin->mutex);
+    set_gui_update_mode(desktop_plugin);
+    pthread_mutex_unlock(&desktop_plugin->mutex); 
+
+
+    error = pthread_create(&tid, NULL, thread_update_values, desktop_plugin);
+
+    if ( error  != 0 ) {
+        fprintf(stderr, "Could not create thread : %d\n", error);
+    }
+    
+    pthread_detach(tid);
 
     return TRUE;
 }
 
-
 void about_dialog() {
 
     GtkWidget *dialog;
@@ -426,7 +448,7 @@ void about_dialog() {
     gtk_text_buffer_insert_pixbuf (GTK_TEXT_BUFFER(text_buffer), &iter, pixbuf);
 
     char tmp_text[1000];
-    sprintf(tmp_text, "\n\nCurrency Exchange Widget version 0.1.1\n\nCurrency Exchange Widget displays the current exchange rate\nof the selected currency.\n\nThis software is released under the GPL license.\n\nSource of exchange rate data: www.xurrency.com\nSource of flag icons: www.icondrawer.com\n\nwww.binop.com\n2010 %lc binop consulting group\n", 0xa9 );
+    sprintf(tmp_text, "\n\nCurrency Exchange Widget version 0.1.2\n\nCurrency Exchange Widget displays the current exchange rate\nof the selected currency.\n\nThis software is released under the GPL license.\n\nSource of exchange rate data: www.xurrency.com\nSource of flag icons: www.icondrawer.com\n\nwww.binop.com\n2010 %lc binop consulting group\n", 0xa9 );
 
 
     gtk_text_buffer_insert(text_buffer, &iter, tmp_text, -1);
@@ -438,21 +460,3 @@ void about_dialog() {
     gtk_widget_destroy(dialog);
 }
 
-
-gboolean
-connection_change( ConIcConnection *connection,   ConIcConnectionEvent *event, CurrencyWidget *desktop_plugin )
-{
-    ConIcConnectionStatus status = con_ic_connection_event_get_status(event);
-    
-    if ( status == CON_IC_STATUS_CONNECTED ) {
-            pthread_mutex_lock(&desktop_plugin->mutex);
-            get_values_json(&desktop_plugin->curset);
-            if ( desktop_plugin->curset->not_updated == 1 ) {
-                strcpy(desktop_plugin->curset->active_currency->value, "offline");
-            }
-            update_ui(desktop_plugin);
-            pthread_mutex_unlock(&desktop_plugin->mutex);
-    }
-    
-    return TRUE;
-}
