@@ -24,7 +24,6 @@
 #include "cJSON.h"
 #include "lib-currency-home-widget.h"
 
-
 extern char * get_html(char *url);
 extern char *currencies[];
 char *get_icon_string(char *st);
@@ -33,7 +32,7 @@ void get_values_json(currency_set **p) {
 
         char *json_string;
         char url[50];    
-        
+    
         strcpy((*p)->name, currencies[(*p)->type - 1]);
         strcpy((*p)->active_currency->name, currencies[(*p)->active_currency->type - 1]);
 
@@ -57,7 +56,7 @@ void get_values_json(currency_set **p) {
             char *tmp = get_icon_string((*p)->active_currency->name);
             strcpy((*p)->active_currency->icon, tmp);
             free(tmp);
-        
+            free(html);
             return;
         }
 
@@ -72,13 +71,15 @@ void get_values_json(currency_set **p) {
         st[7] = '\0'; 
         strncpy((*p)->active_currency->value, st, 20);
 
+
         // set the icon path
 
         char *tmp = get_icon_string((*p)->active_currency->name);
         strcpy((*p)->active_currency->icon, tmp);
+        cJSON_Delete(root);
         free(tmp);
+        free(html);
         time(&(*p)->last_update);
-
 }
 
 char *get_icon_string(char *cur) {
